Unit UFMain;

Interface

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Menus, ExtCtrls, USTreeNodes, USortables, UTFInsertDlg,
  StdCtrls;

Type
  TFMain = Class(TForm)
    MainMenu : TMainMenu;

    MMnuFile : TMenuItem;
    MMnuExit : TMenuItem;

    MMnuEdit      : TMenuItem;
    MnuInsertNode : TMenuItem;
    MnuDeleteNode : TMenuItem;
    MnuDivider1   : TMenuItem;
    MnuFindNode   : TMenuItem;

    MMnuTree        : TMenuItem;
    MnuRandomFill   : TMenuItem;
    MnuRandomValues : TMenuItem;
    MnuNumbers      : TMenuItem;
    MnuLetters      : TMenuItem;
    MnuDivider2     : TMenuItem;
    MnuActualize    : TMenuItem;
    MnuDivider3     : TMenuItem;
    MnuInit         : TMenuItem;
    MnuDivider4     : TMenuItem;
    MnuCheckBalance : TMenuItem;

    StatusBar : TStatusBar;

    Procedure MMnuExitClick(Sender : TObject);
    Procedure CheckSender(Sender : TObject);
    Procedure MnuInsertNodeClick(Sender : TObject);
    Procedure MnuInitClick(Sender : TObject);
    Procedure FormCreate(Sender : TObject);
    Procedure MnuDeleteNodeClick(Sender : TObject);
    Procedure MnuFindNodeClick(Sender : TObject);
    Procedure MnuRandomFillClick(Sender : TObject);
    procedure MnuActualizeClick(Sender: TObject);
    procedure MnuCheckBalanceClick(Sender: TObject);
  End;

Var
  FMain : TFMain;
  STree : TSTreeNodes;

Implementation

{$R *.DFM}


Procedure TFMain.MMnuExitClick;
Begin
  Close;
End;


Procedure TFMain.CheckSender;
Begin
  TMenuItem(Sender).Checked := True;
End;


Procedure TFMain.MnuInsertNodeClick;
Var
  NewString : TString;

Begin
  FInsertDlg := TFInsertDlg.Create(Self);

  FInsertDlg.Caption := 'Knoten Einfgen';

  If FInsertDlg.ShowModal = mrOK Then Begin
    NewString := TString.Create;

    NewString.Value := FInsertDlg.EdValue.Text;

    If Not Assigned(STree.Search(NewString)) Then Begin
      STree.Insert(NewString);
    End
    Else Begin
      MessageDlg('Der Knoten konnte nicht eingefgt werden, da ein Knoten mit dem Inhalt schon vorhanden ist.', mtError, [mbOk], 0);
    End;
  End;

  FInsertDlg.Free;
End;


Procedure TFMain.MnuInitClick;
Begin
  If Assigned(STree) Then Begin
    STree.DestroyNodes;
    STree.Free;
  End;

  STree := TSTreeNodes.Create;

  STree.Canvas       := FMain.Canvas;
  STree.CanvasWidth  := FMain.Width;
  STree.CanvasHeight := FMain.Height;

  STree.Draw(STree.Root);
End;


Procedure TFMain.FormCreate;
Begin
  Randomize;

  MnuInitClick(Self);
End;


Procedure TFMain.MnuDeleteNodeClick;
Var
  DelString : TString;

Begin
  FInsertDlg := TFInsertDlg.Create(Self);

  FInsertDlg.Caption := 'Knoten Lschen';

  If FInsertDlg.ShowModal = mrOK Then Begin
    DelString := TString.Create;

    DelString.Value := FInsertDlg.EdValue.Text;

    If Not STree.Delete(DelString) Then Begin
      MessageDlg('Der Knoten konnte nicht gelscht werden, da ein Knoten mit dem Inhalt nicht vorhanden ist.', mtError, [mbOk], 0);
    End;
  End;

  FInsertDlg.Free;
End;


Procedure TFMain.MnuFindNodeClick(Sender: TObject);
Var
  FindString : TString;

Begin
  FInsertDlg := TFInsertDlg.Create(Self);

  FInsertDlg.Caption := 'Knoten Suchen';

  If FInsertDlg.ShowModal = mrOK Then Begin
    FindString := TString.Create;

    FindString.Value := FInsertDlg.EdValue.Text;

    If (STree.Search(FindString) <> Nil) Then Begin
      MessageDlg('Der Knoten mit dem gesuchten Inhalt ist im Baum vorhanden.', mtInformation, [mbOk], 0);
    End
    Else Begin
      MessageDlg('Der Knoten mit dem gesuchten Inhalt ist nicht im Baum vorhanden.', mtInformation, [mbOk], 0);
    End;
  End;

  FInsertDlg.Free;
End;


Procedure TFMain.MnuRandomFillClick;
Var
  CntrChars,
  CntrNewStrings : Byte;

  NewString : TString;

Begin
  CntrNewStrings := 0;

  While CntrNewStrings < 15 Do Begin
    NewString := TString.Create;

    NewString.Value := '';

    For CntrChars := 1 To 2 Do
      NewString.Value := NewString.Value + Chr(Random(26) + 65);

    If Not Assigned(STree.Search(NewString)) Then Begin
      STree.Insert(NewString);
      Inc(CntrNewStrings);
    End;
  End;
End;


Procedure TFMain.MnuActualizeClick;
Begin
  STree.Draw(STree.Root);
End;


Procedure TFMain.MnuCheckBalanceClick;
Begin
  STree.Draw(STree.Root, True, True);
End;


End.
